@@ -92,8 +92,11 @@ class PWidgetBase(PSignalBase):
         SIGNAL_FOCUS_OUT,
     )
     
+    setup_signals = None
+    
     def __init__(self, manager, parent, geometry=None, font=None):
         self.manager = manager
+        manager.all_widgets.append(self)
         self.parent = parent
         if geometry is not None:
             self.set_geometry(*geometry)
@@ -260,7 +263,7 @@ class PWidgetBase(PSignalBase):
         raise NotImplementedError
 
 
-class PDialogBase(Singleton):
+class PAppDialogBase(Singleton):
     
     def _init(self, parent):
         self.parent = parent
@@ -280,7 +283,7 @@ class PMessageBoxMeta(type):
         setattr(cls, 'answer_map', answer_map)
 
 
-class PMessageBoxBase(PDialogBase, metaclass=PMessageBoxMeta):
+class PMessageBoxBase(PAppDialogBase, metaclass=PMessageBoxMeta):
     
     question_map = {}
     
@@ -335,7 +338,7 @@ class PMessageBoxBase(PDialogBase, metaclass=PMessageBoxMeta):
                               ANSWER_YES, ANSWER_NO, ANSWER_CANCEL)
 
 
-class PFileDialogBase(PDialogBase):
+class PFileDialogBase(PAppDialogBase):
     
     def choose_directory(self, curdir):
         """Display select folder dialog and return chosen folder.
@@ -377,7 +380,7 @@ class PFileDialogBase(PDialogBase):
         return self._save_filename(path, self._translate_filter(filter), self._translate_filter_item(*selected_filter))
 
 
-class PAboutDialogBase(PDialogBase):
+class PAboutDialogBase(PAppDialogBase):
     """Base class for about dialogs.
     """
     
@@ -513,7 +516,130 @@ class PTopWindowBase(PWidgetBase):
         raise NotImplementedError
 
 
-class PApplicationBase(PSignalBase):
+class PManagerBase:
+    """Base class for non-widget object that manages widgets.
+    
+    Objects of this type can be the ``manager`` in a widget's constructor. The main
+    function of widget managers is to implement event handlers for the widgets
+    they manage.
+    """
+    
+    def __init__(self):
+        self.all_widgets = []
+        self.sub_managers = []
+    
+    # Note: every non-abstract subclass of this class must either initialize the manager attribute
+    # in its constructor, or override the app and controller properties (as PApplication does,
+    # to just return self)
+    
+    @property
+    def app(self):
+        app = self.manager
+        while not isinstance(app, PApplicationBase):
+            app = app.manager
+        return app
+    
+    @property
+    def controller(self):
+        controller = self.manager
+        while not isinstance(controller, PControllerBase):
+            controller = controller.manager
+        return controller
+    
+    handler_prefix = None
+    
+    def setup_signals(self):
+        for widget in self.all_widgets:
+            setup = widget.setup_signals
+            if callable(setup):
+                setup()
+        for manager in self.sub_managers:
+            manager.setup_signals()
+        if self.handler_prefix:
+            setup_signal_handlers(self, self, self.handler_prefix)
+    
+    def do_after_create(self):
+        for manager in self.sub_managers:
+            manager.do_after_create()
+        self.after_create()
+    
+    def do_populate_data(self):
+        for manager in self.sub_managers:
+            manager.check_populate_data()
+        self.check_populate_data()
+    
+    showing = True
+    
+    def check_populate_data(self):
+        if self.showing:
+            self.populate_data()
+    
+    def after_create(self):
+        """Placeholder for derived classes if needed.
+        
+        This method can assume that all widgets are created and available and that
+        all auto-connected signal handlers are connected.
+        """
+        pass
+    
+    def populate_data(self):
+        """Placeholder for derived classes if needed.
+        
+        This method can assume that all widgets are created and available, that
+        all auto-connected signal handlers are connected, and that all after_create
+        processing has been completed.
+        """
+        pass
+
+
+class PControllerBase(PManagerBase):
+    """Base class for widget controllers.
+    
+    Widget controllers are widget managers, but they also control the actual widget
+    creation process. Currently the only controllers are the application object and
+    dialog runners.
+    """
+    
+    def before_create(self):
+        """Placeholder for derived classes if needed.
+        
+        This method is called before any widgets are created or specs are evaluated.
+        Note that we cannot call any widgets or sub-managers because they have not
+        been created or initialized yet. That is why this method is only on the app,
+        not any other widget managers.
+        """
+        pass
+    
+    def create_widgets(self):
+        """This method should create whatever widgets are being managed.
+        
+        Derived classes must override and implement.
+        """
+        raise NotImplementedError
+    
+    def do_create(self):
+        """This method should be called at the proper time to do widget creation.
+        """
+        
+        # Allow custom processing before any widgets are created
+        self.before_create()
+        
+        # Create the widgets
+        self.create_widgets()
+        
+        # This is factored out to allow custom signal setup by toolkits if needed;
+        # we do it before do_after_create to ensure that there are not multiple
+        # references to widgets or managers
+        self.setup_signals()
+        
+        # Allow custom processing after all widgets are created
+        self.do_after_create()
+        
+        # Allow loading of data after everything else is done
+        self.do_populate_data()
+
+
+class PApplicationBase(PControllerBase, PSignalBase):
     """Base class for GUI application.
     """
     
@@ -521,6 +647,8 @@ class PApplicationBase(PSignalBase):
         # SIGNAL_BEFOREQUIT
     )
     
+    handler_prefix = 'app'
+    
     about_dialog_class = None
     message_box_class = None
     file_dialog_class = None
@@ -550,27 +678,23 @@ class PApplicationBase(PSignalBase):
     status_labels = None
     
     def __init__(self, arglist=[]):
+        PControllerBase.__init__(self)
         self.arglist = arglist
     
-    def do_create(self):
-        # Allow custom processing before any widgets are created
-        self.before_create()
-        
+    @property
+    def app(self):
+        return self  # for compatibility with other widget managers/controllers
+    
+    @property
+    def controller(self):
+        return self  # for compatibility with other widget managers/controllers
+    
+    def create_widgets(self):
         self.main_window = self.create_main_widget()
         self.main_window.before_client()
         self.client_widget = self.create_client_widget()
         if self.main_iconfile:
             self.main_window.set_iconfile(self.main_iconfile)
-        
-        # This is factored out to allow custom signal setup by toolkits if needed
-        self.setup_signals()
-        
-        # Allow custom processing after all widgets are created
-        self.after_create()
-    
-    def setup_signals(self):
-        # This does nothing unless the user has added custom signals to the app
-        setup_signal_handlers(self, self, 'app')
     
     @property
     def use_mainwindow(self):
@@ -601,7 +725,7 @@ class PApplicationBase(PSignalBase):
         spec = self.main_widget
         if spec:
             main_window = self.main_window
-            client_widget = widget_from_spec(self, main_window, *spec)
+            client_widget = widget_from_spec(self, main_window, spec)
             main_window.set_client_widget(client_widget)
             return client_widget
         return None
@@ -645,20 +769,6 @@ class PApplicationBase(PSignalBase):
     def exit_app(self):
         self.main_window.exit()
     
-    def before_create(self):
-        """Placeholder for derived classes if needed.
-        
-        This method is called before any widgets are created or specs are evaluated.
-        """
-        pass
-    
-    def after_create(self):
-        """Placeholder for derived classes if needed.
-        
-        This method can assume that all widgets are created and available.
-        """
-        pass
-    
     def accept_close(self):
         """Return False if app should not close based on current state.
         """
