@@ -76,9 +76,9 @@ signal_map = {
     SIGNAL_ACTIVATED: "triggered",
     SIGNAL_CLICKED: "clicked",
     SIGNAL_TOGGLED: "toggled",
-    SIGNAL_SELECTED: "activated",
+    SIGNAL_SELECTED: "currentIndexChanged",
     SIGNAL_LISTVIEWSELECTED: "currentItemChanged",
-    SIGNAL_LISTBOXSELECTED: "currentItemChanged",
+    SIGNAL_LISTBOXSELECTED: "currentRowChanged",
     SIGNAL_CELLSELECTED: "currentCellChanged",
     SIGNAL_TABLECHANGED: "cellChanged",
     SIGNAL_TEXTCHANGED: "textChanged",
@@ -92,6 +92,7 @@ signal_map = {
 }
 
 event_map = {
+    SIGNAL_BUTTONSELECTED: ("sig_buttonSelected", (int,)),
     SIGNAL_CELLCHANGED: ("sig_cellChanged", (int, int)),
     SIGNAL_TEXTSTATECHANGED: ("sig_textStateChanged", ()),  # using selectionChanged above does not work
     SIGNAL_FOCUS_IN: ("sig_focusInEvent", ()),
@@ -455,6 +456,8 @@ class PTopWindow(PQtWidget, qt.QMainWindow, PTopWindowBase):
     def closeEvent(self, event):
         self.do_notify(SIGNAL_QUERYCLOSE)  # a handler here can adjust state to be checked below
         
+        # Note that Qt signals/slots are by default synchronous (and we don't use queued connections),
+        # so ``do_notify`` will cause all handlers to execute before it returns
         if self.can_close():
             self.do_notify(SIGNAL_CLOSING)
             event.accept()
