@@ -167,12 +167,14 @@ QueryCloseEvent, EVT_QUERYCLOSE = NewEvent()
 ClosingEvent, EVT_CLOSING = NewEvent()
 ShownEvent, EVT_SHOWN = NewEvent()
 HiddenEvent, EVT_HIDDEN = NewEvent()
+ButtonSelectedEvent, EVT_BUTTON_SELECTED = NewEvent()
 CellSelectedEvent, EVT_CELL_SELECTED = NewEvent()
 CellChangedEvent, EVT_CELL_CHANGED = NewEvent()
 TextModChangedEvent, EVT_TEXT_MOD_CHANGED = NewEvent()
 TextStateChangedEvent, EVT_TEXT_STATE_CHANGED = NewEvent()
 
 custom_map = {
+    SIGNAL_BUTTONSELECTED: ButtonSelectedEvent,
     SIGNAL_CELLSELECTED: CellSelectedEvent,
     SIGNAL_CELLCHANGED: CellChangedEvent,
     SIGNAL_TEXTMODCHANGED: TextModChangedEvent,
@@ -188,6 +190,7 @@ event_map = {
     SIGNAL_CLICKED: wx.EVT_BUTTON,
     SIGNAL_TOGGLED: wx.EVT_CHECKBOX,
     SIGNAL_SELECTED: wx.EVT_COMBOBOX,
+    SIGNAL_BUTTONSELECTED: EVT_BUTTON_SELECTED,
     SIGNAL_FOCUS_IN: wx.EVT_SET_FOCUS,
     SIGNAL_FOCUS_OUT: wx.EVT_KILL_FOCUS,
     SIGNAL_LISTBOXSELECTED: wx.EVT_LISTBOX,
@@ -235,6 +238,12 @@ def wx_selected_wrapper(self, target):
     return wrapper
 
 
+def wx_buttonselected_wrapper(self, target):
+    def wrapper(event):
+        target(self.current_index())
+    return wrapper
+
+
 def wx_listviewselected_wrapper(self, target):
     def wrapper(event):
         # wx doesn't use the "list of lists" model so we have to use our current item method
@@ -244,7 +253,7 @@ def wx_listviewselected_wrapper(self, target):
 
 def wx_listboxselected_wrapper(self, target):
     def wrapper(event):
-        target(self[self.GetSelection()])
+        target(self.GetSelection())
     return wrapper
 
 
@@ -283,6 +292,7 @@ wrapper_map = {
     SIGNAL_CLICKED: wx_plain_wrapper,
     SIGNAL_TOGGLED: wx_toggled_wrapper,
     SIGNAL_SELECTED: wx_selected_wrapper,
+    SIGNAL_BUTTONSELECTED: wx_buttonselected_wrapper,
     SIGNAL_FOCUS_IN: wx_plain_wrapper,
     SIGNAL_FOCUS_OUT: wx_plain_wrapper,
     SIGNAL_LISTVIEWSELECTED: wx_listviewselected_wrapper,
